from __future__ import annotations

import io
import time
from pathlib import Path

import googleapiclient.discovery
import magic
from google.oauth2.service_account import Credentials
from googleapiclient.http import MediaFileUpload, MediaIoBaseDownload

SCOPES = [
    "https://www.googleapis.com/auth/drive",
]


class DriveService:
    # NOTE:
    # https://googleapis.github.io/google-api-python-client/docs/dyn/drive_v3.files.html
    def __init__(self, service_account_file=None, service_account_info=None):
        if service_account_file:
            creds = Credentials.from_service_account_file(
                service_account_file, scopes=SCOPES
            )
        elif service_account_info:
            creds = Credentials.from_service_account_info(
                service_account_info, scopes=SCOPES
            )
        else:
            raise ValueError(
                "Either service_account_file or service_account_info must be provided."
            )
        self.client = googleapiclient.discovery.build(
            "drive", "v3", credentials=creds, cache_discovery=False
        )

    def get_shared_drives(self):
        results = self.client.drives().list(pageSize=1000).execute()
        drives = results.get("drives", [])
        return drives

    def create_folder(self, name: str, folder_id: str) -> str:
        file_metadata = {
            "name": name,
            "mimeType": "application/vnd.google-apps.folder",
            "parents": [folder_id],
        }
        folder = (
            self.client.files()
            .create(body=file_metadata, supportsAllDrives=True)
            .execute()
        )

        return folder.get("id")

    def get_files(self, parent_folder_id: str, share_drive_id: str) -> list:
        try:
            list_ = []
            query = (
                "'%s' in parents and mimeType != 'application/vnd.google-apps.folder' \
                and mimeType != 'application/vnd.google-apps.shortcut' \
                and trashed=false"
                % parent_folder_id
            )
            results = (
                self.client.files()
                .list(
                    q=query,
                    driveId=share_drive_id,
                    corpora="drive",
                    includeItemsFromAllDrives=True,
                    supportsAllDrives=True,
                    pageSize=1000,
                    fields="nextPageToken, files(name,id)",
                )
                .execute()
            )
            items = results.get("files", [])
            for item in items:
                item_name = item["name"]
                list_.append({"name": item_name, "id": item["id"]})
        except Exception as e:
            raise e
        return list_

    def get_folders(self, parent_folder_id: str, share_drive_id: str) -> list:
        try:
            list_ = []
            query = (
                "'%s' in parents and mimeType = 'application/vnd.google-apps.folder' \
                and trashed=false"
                % parent_folder_id
            )
            results = (
                self.client.files()
                .list(
                    q=query,
                    driveId=share_drive_id,
                    corpora="drive",
                    includeItemsFromAllDrives=True,
                    supportsAllDrives=True,
                    pageSize=1000,
                    fields="nextPageToken, files(name,id)",
                )
                .execute()
            )
            items = results.get("files", [])
            for item in items:
                item_name = item["name"]
                list_.append({"name": item_name, "id": item["id"]})
        except Exception as e:
            raise e
        return list_

    def get_child_items(self, parent_folder_id: str, share_drive_id: str) -> list:
        try:
            list_ = []
            query = f'"{parent_folder_id}" in parents and trashed=false'
            results = (
                self.client.files()
                .list(
                    q=query,
                    driveId=share_drive_id,
                    corpora="drive",
                    includeItemsFromAllDrives=True,
                    supportsAllDrives=True,
                    pageSize=1000,
                    fields="nextPageToken, files(name,id)",
                )
                .execute()
            )
            items = results.get("files", [])
            for item in items:
                list_.append({"name": item["name"], "id": item["id"]})
        except Exception as e:
            raise e
        return list_

    def download_file(self, path: str, file_id: str):
        try:
            request = self.client.files().get_media(fileId=file_id)
            fh = io.FileIO(path, mode="wb")
            downloader = MediaIoBaseDownload(fh, request)
            done = False
            while done is False:
                _, done = downloader.next_chunk()
        except Exception as e:
            raise e

    def upload_file(self, file_path: str, folder_id) -> str:
        try:
            mime_type = magic.from_file(file_path, mime=True)
            file_metadata = {
                "name": Path(file_path).name,
                "mimeType": mime_type,
                "parents": [folder_id],
            }
            media = MediaFileUpload(file_path, mimetype=mime_type)
            file = (
                self.client.files()
                .create(
                    body=file_metadata,
                    media_body=media,
                    fields="id",
                    supportsAllDrives=True,
                )
                .execute()
            )
            file_id = file.get("id")
        except Exception as e:
            raise e
        return file_id

    def delete_file(self, file_id: str):
        try:
            self.client.files().delete(fileId=file_id, supportsAllDrives=True).execute()
        except Exception as e:
            raise e

    def rename_file(self, file_id: str, rename: str):
        try:
            file = {"name": rename}
            r = (
                self.client.files()
                .update(fileId=file_id, body=file, supportsAllDrives=True)
                .execute()
            )
            # WARNING:
            # update func is async,
            # It takes time to change the file name.
            for _ in range(10):
                check = (
                    self.client.files()
                    .get(fileId=file_id, fields="name", supportsAllDrives=True)
                    .execute()
                )
                if check["name"] == rename:
                    return r
                time.sleep(0.5)
            raise TimeoutError(f"Failed to rename file {file_id}")
        except Exception as e:
            raise e

    def move_file(self, file_id: str, src_folder_id: str, dst_folder_id: str):
        return (
            self.client.files()
            .update(
                fileId=file_id,
                addParents=dst_folder_id,
                removeParents=src_folder_id,
                supportsAllDrives=True,
            )
            .execute()
        )



from __future__ import annotations

import re
from pathlib import Path

import magic
import requests


class Chatwork:
    BASE_URL = "https://api.chatwork.com/v2/"

    def __init__(self, token: str):
        """Set ChatWork's API token.

        Args:
            token: ChatWork API token.
        """
        self._token = token

    def set_token(self, token: str):
        """Set another ChatWork's API token.

        Args:
            token: ChatWork API token.
        """
        self._token = token

    def post_messages(self, message: str, room_id: int) -> dict | None:
        """Add new message to the chat.

        Args:
            message: message body.
            room_id: ChatWork room id.

        Returns:
            JSON Format response as a dict.

        Raises:
            Raises stored :class:`HTTPError`, if one occurred.
        """
        res = requests.post(
            self._make_url("rooms/{}/messages".format(room_id)),
            headers=self._make_headers(),
            params=self._make_body(message=message),
        )
        return self._check_res(res, dict)

    def post_file(self, message: str, room_id: int, file_path: str) -> dict | None:
        """Add new message to the chat.

        Args:
            message: message body.
            room_id: ChatWork room id.
            file_path: File path to be uploaded.

        Returns:
            JSON Format response as a dict.

        Raises:
            Raises stored :class:`HTTPError`, if one occurred.
        """
        res = requests.post(
            self._make_url("rooms/{}/files".format(room_id)),
            headers=self._make_headers(),
            params=self._make_body(message=message),
            files=self._make_files(file_path),
        )
        return self._check_res(res, dict)

    def post_task(
        self,
        message: str,
        room_id: int,
        to_ids: str,
    ) -> dict | None:
        """Add new message to the chat.

        Args:
            message: message body.
            room_id: ChatWork room id.
            to_ids: assingned user ids, separated by comma.

        Returns:
            JSON Format response as a dict.

        Raises:
            Raises stored :class:`HTTPError`, if one occurred.
        """
        res = requests.post(
            self._make_url("rooms/{}/tasks".format(room_id)),
            headers=self._make_headers(),
            params=self._make_body(message=message, to_ids=to_ids),
        )
        return self._check_res(res, dict)

    def get_contacts(self) -> list | None:
        """Get contacts.

        Args:
            No arguments.

        Returns:
            JSON Format response as a list.

        Raises:
            Raises stored :class:`HTTPError`, if one occurred.
        """
        res = requests.get(
            self._make_url("contacts"),
            headers=self._make_headers(),
        )
        return self._check_res(res, list)

    def get_messages(self, room_id: int, force: bool = False) -> list | None:
        """Get new messages from a room.

        If you set force=True, you can get older messages.

        Args:
            room_id: ChatWork room id.
            force (optional): Flag which forces to get 100 newest entries
                regardless of previous calls.

        Returns:
            JSON Format response as a list.

        Raises:
            Raises stored :class:`HTTPError`, if one occurred.
        """
        forceflg = "?force=1" if force else "?force=0"
        res = requests.get(
            self._make_url("rooms/{}/messages".format(room_id) + forceflg),
            headers=self._make_headers(),
        )
        return self._check_res(res, list)

    def _make_url(self, endpoint: str) -> str:
        return self.BASE_URL + endpoint

    def _make_headers(self):
        return {"X-ChatWorkToken": self._token}

    def _make_body(
        self,
        message: str,
        to_ids: str | None = None,
    ):
        payload = {}
        if message:
            payload["body"] = message
        if to_ids:
            payload["to_ids"] = to_ids
        return payload

    def _make_files(self, file_path: str) -> dict:
        file = Path(file_path)
        mimetype = magic.from_file(file, mime=True)
        with file.open("rb") as f:
            data = f.read()
        return {"file": (file.name, data, mimetype)}

    def _check_res(self, res, deftype):
        if res.ok:
            return self._check_status_code(res, deftype)
        else:
            message = res.json()
            raise Exception(message["errors"])

    def _check_status_code(self, res, deftype):
        if res.status_code == 200:
            return res.json()
        elif res.status_code == 204:
            return deftype()
        else:
            res.raise_for_status()

    def _make_firstline(self, to_names: list[str] | None):
        regex = "[ 　]"
        first_line = ""
        if to_names:
            contacts = self.get_contacts()
            if contacts:
                for to_name in to_names:
                    for contact in contacts:
                        if re.sub(regex, "", contact["name"]) == re.sub(
                            regex, "", to_name
                        ):
                            first_line += f"[To:{contact['account_id']}]{to_name}さん"
                            break
        return first_line

    def format_message(
        self,
        message: str,
        title: str = "From Python",
        to_names: list[str] | None = None,
    ):
        format_message = f"""
        {self._make_firstline(to_names)}
        [info][title]{title}[/title]{message}[/info]
        """
        return format_message



import boto3


class ParameterStore:
    def __init__(self, region="ap-northeast-1"):
        self.region = region

    def get(self, key):
        try:
            ssm = boto3.client(
                "ssm",
                region_name=self.region,
                endpoint_url=f"https://ssm.{self.region}.amazonaws.com",
            )
            response = ssm.get_parameters(
                Names=[
                    key,
                ],
                WithDecryption=True,
            )
        except Exception as e:
            raise e
        return response["Parameters"][0]["Value"]



import os

from selenium.webdriver.chrome.service import Service
from selenium import webdriver


def setup_driver(is_lambda=True, is_headless=True):
    options = webdriver.ChromeOptions()
    if is_headless:
        options.add_argument("--headless")
    options.add_argument("--no-sandbox")
    options.add_argument("--disable-gpu")
    options.add_argument("--window-size=1920x1080")
    options.add_argument("--single-process")
    options.add_argument("--disable-dev-shm-usage")
    options.add_argument("--disable-dev-tools")
    options.add_argument("--no-zygote")
    if is_lambda:
        dl_path = "/tmp/downloads"
    else:
        dl_path = "./downloads"
    os.makedirs(dl_path, exist_ok=True)
    options.add_experimental_option(
        "prefs",
        {"download.default_directory": dl_path, "download.prompt_for_download": False},
    )
    if is_lambda:
        options.binary_location = "/opt/chrome/chrome"
        chrome = webdriver.Chrome(
            service=Service("/opt/chromedriver"),
            options=options,
        )
    else:
        chrome = webdriver.Chrome(options=options)

    chrome.implicitly_wait(15)
    return chrome



import json

import requests


class Slack:
    def __init__(self, web_hook_url):
        self.web_hook_url = web_hook_url

    def post(self, msg, username="Notification-Bot"):
        requests.post(
            self.web_hook_url,
            data=json.dumps(
                {
                    "text": msg,
                    "username": username,
                    "icon_emoji": ":smile_cat:",
                    "link_names": 1,
                }
            ),
        )



from __future__ import annotations

import googleapiclient.discovery
from google.oauth2.service_account import Credentials

SCOPES = [
    "https://www.googleapis.com/auth/spreadsheets",
]


class SheetService:
    # NOTE:
    # https://googleapis.github.io/google-api-python-client/docs/dyn/sheets_v4.html
    def __init__(self, service_account_file=None, service_account_info=None):
        if service_account_file:
            creds = Credentials.from_service_account_file(
                service_account_file, scopes=SCOPES
            )
        elif service_account_info:
            creds = Credentials.from_service_account_info(
                service_account_info, scopes=SCOPES
            )
        else:
            raise ValueError(
                "Either service_account_file or service_account_info must be provided."
            )
        self.client = googleapiclient.discovery.build(
            "sheets", "v4", credentials=creds, cache_discovery=False
        )

    def get_values(
        self, ss_id: str, ranges: list[str], major_dimension: str = "ROWS"
    ):
        """Get values from a sheet.

        Args:
            ss_id: Google Spreadsheet ID.
            ranges: list of Ranges.
            major_dimension: Either "ROWS" or "COLUMNS".

        Returns:
            JSON Format response as a dict.

        Examples:
            >>> get_values("sn", "Sheet1!A1:B5", "ROWS")

                [['a1', 'b1'], ['a2', 'b2'], ['a3', 'b3'], ['a4', 'b4'], ['a5', 'b5']]}

            >>> get_values("sn", "Sheet1!A1:B5", "COLUMNS")

                [['a1', 'a2', 'a3', 'a4', 'a5'], ['b1', 'b2', 'b3', 'b4', 'b5']]}

        """
        sheet = self.client.spreadsheets()
        values = (
            sheet.values()
            .batchGet(
                spreadsheetId=ss_id,
                ranges=ranges,
                majorDimension=major_dimension,
            )
            .execute()
        )

        return values

    def _make_range(
        self, sheet_name: str, values: list, start_row: int, start_col: int
    ):
        end_row = start_row + len(values) - 1
        end_col = start_col + len(values[0]) - 1
        range = f"{sheet_name}!R{start_row}C{start_col}:R{end_row}C{end_col}"
        return range

    def post_values(
        self,
        ss_id: str,
        values: list,
        sheet_name: str,
        major_dimension: str = "ROWS",
        start_row: int = 1,
        start_col: int = 1,
        range: str | None = None,
    ):
        """Post values to a sheet.

        Args:
            ss_id: Google Spreadsheet ID.
            values: List of values to be inserted.
            sheet_name: Name of the sheet.
            major_dimension: Either "ROWS" or "COLUMNS".
            start_row: if range is None, start_row is used to make range.
            start_col: if range is None, start_col is used to make range.
            range: Range of the sheet.

        Returns:
            JSON Format response as a dict.

        """
        sheet = self.client.spreadsheets()
        if range is None:
            range = self._make_range(
                sheet_name=sheet_name,
                values=values,
                start_row=start_row,
                start_col=start_col,
            )
        body = {"range": range, "majorDimension": major_dimension, "values": values}
        r = (
            sheet.values()
            .update(
                spreadsheetId=ss_id,
                body=body,
                range=range,
                valueInputOption="USER_ENTERED",
            )
            .execute()
        )
        return r
